@@ -33,6 +33,9 @@ import remote
 import local
 
 
+VERSION = '0.2.2'
+
+
 def hook_setup(cmd):
     config.setup()
     sys.exit(0)
@@ -40,6 +43,10 @@ def hook_setup(cmd):
 def hook_destroy(cmd):
     local.confirm('Please confirm that you want to destroy your service')
 
+def hook___version(cmd):
+    print 'DotCloud CLI version {0}'.format(VERSION)
+    sys.exit(0)
+
 _export = False
 def hook___export(cmd):
     global _export
@@ -61,7 +68,6 @@ def hook___import(cmd):
 
 def run_remote(cmd):
     r = remote.Remote()
-    l = local.Local()
     handlers = {
             'set_url': r.set_url,
             'run': r.run,
@@ -69,10 +75,11 @@ def run_remote(cmd):
             'pull': r.pull,
             'push': r.push,
             'rsync': r.rsync,
-            'git': l.git,
-            'hg': l.hg,
+            'git': r.git,
+            'hg': r.hg,
+            'upload': r.upload,
             'confirm': local.confirm,
-            'call': lambda x: run_command(x.split(), True),
+            'call': lambda x: run_command(x, True),
             'echo': lambda x: sys.stdout.write('{0}\n'.format(x)),
             'echo_error': lambda x: sys.stderr.write('{0}\n'.format(x))
             }
@@ -80,9 +87,12 @@ def run_remote(cmd):
         c = args[0]
         if len(args) > 1:
             args = tuple(args[1:])
-            handlers[c](*args)
+            ret = handlers[c](*args)
         else:
-            handlers[c]()
+            ret = handlers[c]()
+        if isinstance(ret, int):
+            if ret != 0:
+                utils.die('Abort.')
 
 def sign_request(api_key, method, request):
     (access_key, secret_key) = api_key.split(':')
@@ -102,19 +112,23 @@ def run_command(cmd, internal=False):
     cfg = config.load_user_config()
     url = utils.parse_url(cfg.url)
     req = httplib.HTTPConnection(url['host'], int(url['port'] or 80))
-    cmd = '%20'.join([urllib.quote(c) if c.find(' ') < 0 else '"{0}"'.format(urllib.quote(c)) for c in cmd])
+    cmd = urllib.quote(json.dumps(cmd))
     query = '{0}run?{1}={2}'.format(url['path'], 'c' if internal is True else 'q', cmd)
     headers = sign_request(cfg.apikey, 'GET', query)
     headers.update({
-        'User-Agent': 'dotcloud/cli'
+        'User-Agent': 'dotcloud/cli (version: {0})'.format(VERSION),
+        'X-DotCloud-Version': VERSION
         })
     try:
         req.request('GET', query, headers=headers)
         resp = req.getresponse()
+        info = resp.getheader('X-Dotcloud-Info')
         data = resp.read()
         if _export is True:
             print data
             return
+        if info:
+            utils.warning(info.replace(';', '\n'))
         data = json.loads(data)
         if data['type'] == 'cmd':
             return run_remote(data['data'])
@@ -122,8 +136,8 @@ def run_command(cmd, internal=False):
             print data['data']
     except socket.error, e:
         utils.die('Cannot reach DotCloud service ("{0}").\nPlease check the connectivity and try again.'.format(str(e)))
-    except Exception, e:
-        utils.die('DotCloud service unavailable ("{0}").\nPlease try again later. If the problem persists, send an email to support@dotcloud.com.'.format(e))
+#    except Exception, e:
+#        utils.die('DotCloud service unavailable ("{0}").\nPlease try again later. If the problem persists, send an email to support@dotcloud.com.'.format(e))
     finally:
         try:
             req.close()
